#include <xparameters.h>
#include <xgpio_l.h>
#include <xintc_l.h>
#include <xtmrctr_l.h>
#include <mb_interface.h>


//*****************************************************************************
//* Makrk a memria rshoz s olvasshoz.                                 *
//*****************************************************************************
#define MEM8(addr)   (*(volatile unsigned char *)(addr))
#define MEM16(addr)  (*(volatile unsigned short *)(addr))
#define MEM32(addr)  (*(volatile unsigned long *)(addr))


//*****************************************************************************
//* Defincik a folyadkszint jelz perifrihoz.                            *
//*****************************************************************************
//Sttusz regiszter: 32 bites, csak olvashat
#define LVL_STATUS_REG				0x00
//Megszakts engedlyez regiszter: 32 bites, rhat/olvashat
#define LVL_IE_REG					0x04
//Megszakts flag regiszter: 32 bites, olvashat  s '1' berssal trlhet
#define LVL_IF_REG					0x08

//A folyadkszint jelz perifria megszakts esemnyei.
#define LVL_FULL_IRQ				(1 << 0)
#define LVL_EMPTY_IRQ				(1 << 1)
#define LVL_ERROR_IRQ				(1 << 2)


//*****************************************************************************
//* Htszegmenses dekder.                                                    *
//*****************************************************************************
unsigned char bin2sevenseg[] = {
	0x3f, 0x06, 0x5b, 0x4f, 0x66, 0x6d, 0x7d, 0x07,
	0x7f, 0x6f, 0x77, 0x7c, 0x39, 0x5e, 0x79, 0x71
};


//*****************************************************************************
//* Az idzt megszaktskezel rutinja.                                     *
//*****************************************************************************
volatile unsigned char led_value;
volatile unsigned char led_blink;

void timer_int_handler(void *instancePtr)
{
	unsigned long csr;

	//Meghatrozzuk a LED-ekre kirand j rtket s amennyiben engedlyezett
	//a villogs, akkor azt kirjuk a LED-ekre. Ha a villogs nem engedlyezett,
	//akkor a LED-eket kikapcsoljuk.
	led_value = ~led_value & 0xff;
	if (led_blink)
	{
		XGpio_WriteReg(XPAR_GPIO_LED_DISP_BASEADDR, XGPIO_DATA_OFFSET, led_value);
	}
	else
	{
		XGpio_WriteReg(XPAR_GPIO_LED_DISP_BASEADDR, XGPIO_DATA_OFFSET, 0x00);
	}

	//A megszakts jelzs trlse. A jelzs az idzt kontroll/sttusz
	//regiszterben 1 bersval trlhet.
	//XTmrCtr_GetControlStatusReg()
	//XTmrCtr_SetControlStatusReg()
	csr = XTmrCtr_GetControlStatusReg(XPAR_AXI_TIMER_0_BASEADDR, 0);
	XTmrCtr_SetControlStatusReg(XPAR_AXI_TIMER_0_BASEADDR, 0, csr);
}


//*****************************************************************************
//* A GPIO perifria megszaktskezel rutinja.                               *
//*****************************************************************************
void gpio_int_handler(void *instancePtr)
{
	unsigned char ifr, btn;

	//A megszakts jelzs trlse. A jelzs a GPIO megszakts sttusz
	//regiszterben 1 bersval trlhet.
	//XGpio_ReadReg()
	//XGpio_WriteReg()
	ifr = XGpio_ReadReg(XPAR_GPIO_SW_BTN_BASEADDR, XGPIO_ISR_OFFSET);
	XGpio_WriteReg(XPAR_GPIO_SW_BTN_BASEADDR, XGPIO_ISR_OFFSET, ifr);

	//A GPIO CH2 llapotvltozs megszakts vizsglata. Ha a BTN0 gombot
	//megnyomtk, akkor tiltjuk a LED-ek villogst.
	if (ifr & XGPIO_IR_CH2_MASK)
	{
		btn = XGpio_ReadReg(XPAR_GPIO_SW_BTN_BASEADDR, XGPIO_DATA2_OFFSET);
		if (btn & (1 << 0))
		{
			led_blink = 0;
		}
	}
}


//*****************************************************************************
//* A folyadkszint kijelz perifria megszaktskezel rutinja.              *
//*****************************************************************************
void lvl_int_handler(void *instancePtr)
{
	unsigned char ifr;

	//A megszakts jelzs trlse. A jelzs a perifria megszakts flag
	//regiszterben 1 bersval trlhet.
	ifr = MEM32(XPAR_LVL_INDICATOR_0_BASEADDR + LVL_IF_REG);
	MEM32(XPAR_LVL_INDICATOR_0_BASEADDR + LVL_IF_REG) = ifr;

	//Hibajelzs esetn engedlyezzk a LED-ek villogst.
	if (ifr & LVL_ERROR_IRQ)
	{
		led_blink = 1;
	}
}


//*****************************************************************************
//* Fprogram.                                                                *
//*****************************************************************************
int main()
{
	unsigned long data;

	led_value = 0;
	led_blink = 0;

	//A megszaktskezel rutinok regisztrlsa.
	//XIntc_RegisterHandler()
	XIntc_RegisterHandler(
		XPAR_INTC_0_BASEADDR,
		XPAR_MICROBLAZE_0_AXI_INTC_AXI_TIMER_0_INTERRUPT_INTR,
		(XInterruptHandler)timer_int_handler,
		NULL
	);

	XIntc_RegisterHandler(
		XPAR_INTC_0_BASEADDR,
		XPAR_MICROBLAZE_0_AXI_INTC_GPIO_SW_BTN_IP2INTC_IRPT_INTR,
		(XInterruptHandler)gpio_int_handler,
		NULL
	);

	XIntc_RegisterHandler(
		XPAR_INTC_0_BASEADDR,
		XPAR_MICROBLAZE_0_AXI_INTC_LVL_INDICATOR_0_IRQ_INTR,
		(XInterruptHandler)lvl_int_handler,
		NULL
	);

	//A hasznlt megszaktsok engedlyezse a megszakts vezrlben.
	//XIntc_MasterEnable()
	//XIntc_EnableIntr()
	XIntc_MasterEnable(XPAR_INTC_0_BASEADDR);
	XIntc_EnableIntr(
		XPAR_INTC_0_BASEADDR,
		XPAR_AXI_TIMER_0_INTERRUPT_MASK |
		XPAR_GPIO_SW_BTN_IP2INTC_IRPT_MASK |
		XPAR_LVL_INDICATOR_0_IRQ_MASK
	);

	//A CH2 llapotvltozs megszakts engedlyezse a kapcsol/nyomgomb GPIO perifriban:
	//CH2 flag trlse, CH2 megszakts engedlyezs, globlis megszakts engedlyezs.
	//XGpio_WriteReg()
	XGpio_WriteReg(
		XPAR_GPIO_SW_BTN_BASEADDR,
		XGPIO_ISR_OFFSET,
		XGPIO_IR_CH2_MASK
	);
	XGpio_WriteReg(
		XPAR_GPIO_SW_BTN_BASEADDR,
		XGPIO_IER_OFFSET,
		XGPIO_IR_CH2_MASK
	);
	XGpio_WriteReg(
		XPAR_GPIO_SW_BTN_BASEADDR,
		XGPIO_GIE_OFFSET,
		XGPIO_GIE_GINTR_ENABLE_MASK
	);

	//A hiba megszakts engedlyezse a folyadkszint kijelz perifriban:
	//ERROR flag trlse, ERROR megszakts engedlyezse.
	MEM32(XPAR_LVL_INDICATOR_0_BASEADDR + LVL_IF_REG) = LVL_ERROR_IRQ;
	MEM32(XPAR_LVL_INDICATOR_0_BASEADDR + LVL_IE_REG) = LVL_ERROR_IRQ;

	//A megszaktsok engedlyezse a MicroBlaze processzoron.
	//microblaze_enable_interrupts()
	microblaze_enable_interrupts();

	//A timer LOAD regiszternek belltsa (megszakts 0,25 msodpercenknt).
	//XTmrCtr_SetLoadReg()
	XTmrCtr_SetLoadReg(
		XPAR_AXI_TIMER_0_BASEADDR,
		0,
		XPAR_AXI_TIMER_0_CLOCK_FREQ_HZ / 4
	);

	//A timer alapllapotba lltsa:
	//- a timer lelltsa
	//- a megszakts jelzs trlse
	//- a LOAD regiszter betltse a szmllba
	//XTmrCtr_SetControlStatusReg()
	XTmrCtr_SetControlStatusReg(
		XPAR_AXI_TIMER_0_BASEADDR,
		0,
		XTC_CSR_INT_OCCURED_MASK | XTC_CSR_LOAD_MASK
	);

	//A timer elindtsa:
	//- a timer engedlyezse
	//- a megszakts engedlyezse
	//- automatikus szmll jratlts (periodikus) md
	//- a szmll lefele szmll (0 a vgllapot)
	//XTmrCtr_SetControlStatusReg()
	XTmrCtr_SetControlStatusReg(
		XPAR_AXI_TIMER_0_BASEADDR,
		0,
		XTC_CSR_ENABLE_TMR_MASK | XTC_CSR_ENABLE_INT_MASK |
		XTC_CSR_AUTO_RELOAD_MASK | XTC_CSR_DOWN_COUNT_MASK
	);

	//A vgtelen ciklusban folyamatosan beolvassuk a folyadkszint
	//rtkt s kijelezzk a htszegmenses kijelz DIG0 digitjre.
	for (;;)
	{
		data = MEM32(XPAR_LVL_INDICATOR_0_BASEADDR + LVL_STATUS_REG);
		data = bin2sevenseg[data & 0x0f];
		data |= (1 << 8);
		XGpio_WriteReg(
			XPAR_GPIO_LED_DISP_BASEADDR,
			XGPIO_DATA2_OFFSET,
			data
		);

	}

	return 0;
}
